@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2021 Open Source Applications Foundation.
+ * Copyright (c) 2004-2025 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,8 @@
 #include "calendar.h"
 #include "macros.h"
 
+#include "arg.h"
+
 DECLARE_CONSTANTS_TYPE(UCalendarDateFields)
 DECLARE_CONSTANTS_TYPE(UCalendarDaysOfWeek)
 DECLARE_CONSTANTS_TYPE(UCalendarMonths)
@@ -178,7 +180,7 @@ static PyObject *t_calendar_setTime(t_calendar *self, PyObject *arg)
 {
     UDate date;
 
-    if (!parseArg(arg, "D", &date))
+    if (!parseArg(arg, arg::D(&date)))
     {
         STATUS_CALL(self->object->setTime(date, status));
         Py_RETURN_SELF();
@@ -190,11 +192,10 @@ static PyObject *t_calendar_setTime(t_calendar *self, PyObject *arg)
 static PyObject *t_calendar_isEquivalentTo(t_calendar *self, PyObject *arg)
 {
     Calendar *calendar;
-    int b;
 
-    if (!parseArg(arg, "P", TYPE_ID(Calendar), &calendar))
+    if (!parseArg(arg, arg::P<Calendar>(TYPE_ID(Calendar), &calendar)))
     {
-        b = self->object->isEquivalentTo(*calendar);
+        int b = self->object->isEquivalentTo(*calendar);
         Py_RETURN_BOOL(b);
     }
 
@@ -206,7 +207,7 @@ static PyObject *t_calendar_equals(t_calendar *self, PyObject *arg)
     Calendar *calendar;
     int b;
 
-    if (!parseArg(arg, "P", TYPE_ID(Calendar), &calendar))
+    if (!parseArg(arg, arg::P<Calendar>(TYPE_ID(Calendar), &calendar)))
     {
         STATUS_CALL(b = self->object->equals(*calendar, status));
         Py_RETURN_BOOL(b);
@@ -220,7 +221,7 @@ static PyObject *t_calendar_before(t_calendar *self, PyObject *arg)
     Calendar *calendar;
     int b;
 
-    if (!parseArg(arg, "P", TYPE_ID(Calendar), &calendar))
+    if (!parseArg(arg, arg::P<Calendar>(TYPE_ID(Calendar), &calendar)))
     {
         STATUS_CALL(b = self->object->before(*calendar, status));
         Py_RETURN_BOOL(b);
@@ -234,7 +235,7 @@ static PyObject *t_calendar_after(t_calendar *self, PyObject *arg)
     Calendar *calendar;
     int b;
 
-    if (!parseArg(arg, "P", TYPE_ID(Calendar), &calendar))
+    if (!parseArg(arg, arg::P<Calendar>(TYPE_ID(Calendar), &calendar)))
     {
         STATUS_CALL(b = self->object->after(*calendar, status));
         Py_RETURN_BOOL(b);
@@ -248,7 +249,9 @@ static PyObject *t_calendar_add(t_calendar *self, PyObject *args)
     UCalendarDateFields field;
     int amount;
 
-    if (!parseArgs(args, "ii", &field, &amount))
+    if (!parseArgs(args,
+                   arg::Enum<UCalendarDateFields>(&field),
+                   arg::i(&amount)))
     {
         STATUS_CALL(self->object->add(field, amount, status));
         Py_RETURN_SELF();
@@ -260,14 +263,19 @@ static PyObject *t_calendar_add(t_calendar *self, PyObject *args)
 static PyObject *t_calendar_roll(t_calendar *self, PyObject *args)
 {
     UCalendarDateFields field;
-    int amount, up;
+    int amount;
+    UBool up;
 
-    if (!parseArgs(args, "iB", &field, &up))
+    if (!parseArgs(args,
+                   arg::Enum<UCalendarDateFields>(&field),
+                   arg::B(&up)))
     {
         STATUS_CALL(self->object->roll(field, (UBool) up, status));
         Py_RETURN_SELF();
     }
-    if (!parseArgs(args, "ii", &field, &amount))
+    if (!parseArgs(args,
+                   arg::Enum<UCalendarDateFields>(&field),
+                   arg::i(&amount)))
     {
         STATUS_CALL(self->object->roll(field, amount, status));
         Py_RETURN_SELF();
@@ -282,7 +290,9 @@ static PyObject *t_calendar_fieldDifference(t_calendar *self, PyObject *args)
     UCalendarDateFields field;
     int diff;
 
-    if (!parseArgs(args, "Di", &date, &field))
+    if (!parseArgs(args,
+                   arg::D(&date),
+                   arg::Enum<UCalendarDateFields>(&field)))
     {
         STATUS_CALL(diff = self->object->fieldDifference(date, field, status));
         return PyInt_FromLong(diff);
@@ -301,7 +311,7 @@ static PyObject *t_calendar_setTimeZone(t_calendar *self, PyObject *arg)
 {
     TimeZone *tz;
 
-    if (!parseArg(arg, "P", TYPE_CLASSID(TimeZone), &tz))
+    if (!parseArg(arg, arg::P<TimeZone>(TYPE_CLASSID(TimeZone), &tz)))
     {
         self->object->setTimeZone(*tz); /* copied */
         Py_RETURN_SELF();
@@ -326,9 +336,9 @@ static PyObject *t_calendar_isLenient(t_calendar *self)
 
 static PyObject *t_calendar_setLenient(t_calendar *self, PyObject *arg)
 {
-    int b;
+    UBool b;
 
-    if (!parseArg(arg, "b", &b))
+    if (!parseArg(arg, arg::b(&b)))
     {
         self->object->setLenient(b);
         Py_RETURN_SELF();
@@ -351,7 +361,7 @@ static PyObject *t_calendar_setFirstDayOfWeek(t_calendar *self, PyObject *arg)
 {
     UCalendarDaysOfWeek day;
 
-    if (!parseArg(arg, "i", &day))
+    if (!parseArg(arg, arg::Enum<UCalendarDaysOfWeek>(&day)))
     {
         self->object->setFirstDayOfWeek(day);
         Py_RETURN_SELF();
@@ -370,7 +380,7 @@ static PyObject *t_calendar_setMinimalDaysInFirstWeek(t_calendar *self,
 {
     int days;
 
-    if (!parseArg(arg, "i", &days))
+    if (!parseArg(arg, arg::i(&days)))
     {
         self->object->setMinimalDaysInFirstWeek(days);
         Py_RETURN_SELF();
@@ -383,7 +393,7 @@ static PyObject *t_calendar_getMinimum(t_calendar *self, PyObject *arg)
 {
     UCalendarDateFields field;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
         return PyInt_FromLong(self->object->getMinimum(field));
 
     return PyErr_SetArgsError((PyObject *) self, "getMinimum", arg);
@@ -393,7 +403,7 @@ static PyObject *t_calendar_getMaximum(t_calendar *self, PyObject *arg)
 {
     UCalendarDateFields field;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
         return PyInt_FromLong(self->object->getMaximum(field));
 
     return PyErr_SetArgsError((PyObject *) self, "getMaximum", arg);
@@ -403,7 +413,7 @@ static PyObject *t_calendar_getGreatestMinimum(t_calendar *self, PyObject *arg)
 {
     UCalendarDateFields field;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
         return PyInt_FromLong(self->object->getGreatestMinimum(field));
 
     return PyErr_SetArgsError((PyObject *) self, "getGreatestMinimum", arg);
@@ -414,7 +424,7 @@ static PyObject *t_calendar_getActualMinimum(t_calendar *self, PyObject *arg)
     UCalendarDateFields field;
     int actual;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
     {
         STATUS_CALL(actual = self->object->getActualMinimum(field, status));
         return PyInt_FromLong(actual);
@@ -427,7 +437,7 @@ static PyObject *t_calendar_getLeastMaximum(t_calendar *self, PyObject *arg)
 {
     UCalendarDateFields field;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
         return PyInt_FromLong(self->object->getLeastMaximum(field));
 
     return PyErr_SetArgsError((PyObject *) self, "getLeastMaximum", arg);
@@ -438,7 +448,7 @@ static PyObject *t_calendar_getActualMaximum(t_calendar *self, PyObject *arg)
     UCalendarDateFields field;
     int actual;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
     {
         STATUS_CALL(actual = self->object->getActualMaximum(field, status));
         return PyInt_FromLong(actual);
@@ -452,7 +462,7 @@ static PyObject *t_calendar_get(t_calendar *self, PyObject *arg)
     UCalendarDateFields field;
     int value;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
     {
         STATUS_CALL(value = self->object->get(field, status));
         return PyInt_FromLong(value);
@@ -466,7 +476,7 @@ static PyObject *t_calendar_isSet(t_calendar *self, PyObject *arg)
     UCalendarDateFields field;
     int b;
 
-    if (!parseArg(arg, "i", &field))
+    if (!parseArg(arg, arg::Enum<UCalendarDateFields>(&field)))
     {
         b = self->object->isSet(field);
         Py_RETURN_BOOL(b);
@@ -477,33 +487,34 @@ static PyObject *t_calendar_isSet(t_calendar *self, PyObject *arg)
 
 static PyObject *t_calendar_set(t_calendar *self, PyObject *args)
 {
-    int year, month, date, hour, minute, second;
-    int field, value;
+    int year, month, date, hour, minute, second, value;
+    UCalendarDateFields field;
     switch (PyTuple_Size(args)) {
       case 2:
-        if (!parseArgs(args, "ii", &field, &value))
+        if (!parseArgs(args,
+                       arg::Enum<UCalendarDateFields>(&field),
+                       arg::i(&value)))
         {
             self->object->set((UCalendarDateFields) field, value);
             Py_RETURN_SELF();
         }
         break;
       case 3:
-        if (!parseArgs(args, "iii", &year, &month, &date))
+        if (!parseArgs(args, arg::i(&year), arg::i(&month), arg::i(&date)))
         {
             self->object->set(year, month, date);
             Py_RETURN_SELF();
         }
         break;
       case 5:
-        if (!parseArgs(args, "iiiii", &year, &month, &date, &hour, &minute))
+        if (!parseArgs(args, arg::i(&year), arg::i(&month), arg::i(&date), arg::i(&hour), arg::i(&minute)))
         {
             self->object->set(year, month, date, hour, minute);
             Py_RETURN_SELF();
         }
         break;
       case 6:
-        if (!parseArgs(args, "iiiiii", &year, &month, &date, &hour, &minute,
-                       &second))
+        if (!parseArgs(args, arg::i(&year), arg::i(&month), arg::i(&date), arg::i(&hour), arg::i(&minute), arg::i(&second)))
         {
             self->object->set(year, month, date, hour, minute, second);
             Py_RETURN_SELF();
@@ -523,7 +534,7 @@ static PyObject *t_calendar_clear(t_calendar *self, PyObject *args)
         self->object->clear();
         Py_RETURN_SELF();
       case 1:
-        if (!parseArgs(args, "i", &field))
+        if (!parseArgs(args, arg::Enum<UCalendarDateFields>(&field)))
         {
             self->object->clear(field);
             Py_RETURN_SELF();
@@ -563,7 +574,7 @@ static PyObject *t_calendar_getLocale(t_calendar *self, PyObject *args)
                                                      status));
         return wrap_Locale(locale);
       case 1:
-        if (!parseArgs(args, "i", &type))
+        if (!parseArgs(args, arg::Enum<ULocDataLocaleType>(&type)))
         {
             STATUS_CALL(locale = self->object->getLocale(type, status));
             return wrap_Locale(locale);
@@ -584,7 +595,7 @@ static PyObject *t_calendar_getLocaleID(t_calendar *self, PyObject *args)
         STATUS_CALL(id = self->object->getLocaleID(ULOC_VALID_LOCALE, status));
         return PyString_FromString(id);
       case 1:
-        if (!parseArgs(args, "i", &type))
+        if (!parseArgs(args, arg::Enum<ULocDataLocaleType>(&type)))
         {
             STATUS_CALL(id = self->object->getLocaleID(type, status));
             return PyString_FromString(id);
@@ -615,7 +626,7 @@ static PyObject *t_calendar_setTemporalMonthCode(t_calendar *self, PyObject *arg
 {
     charsArg code;
 
-    if (!parseArg(arg, "n", &code))
+    if (!parseArg(arg, arg::n(&code)))
     {
         STATUS_CALL(self->object->setTemporalMonthCode(code, status));
         Py_RETURN_SELF();
@@ -640,22 +651,21 @@ static PyObject *t_calendar_createInstance(PyTypeObject *type, PyObject *args)
           return wrap_Calendar(calendar);
       }
       case 1:
-        if (!parseArgs(args, "P", TYPE_CLASSID(TimeZone), &tz))
+        if (!parseArgs(args, arg::P<TimeZone>(TYPE_CLASSID(TimeZone), &tz)))
         {
             STATUS_CALL(calendar = Calendar::createInstance(*tz, status));
             return wrap_Calendar(calendar);
         }
-        if (!parseArgs(args, "P", TYPE_CLASSID(Locale), &locale))
+        if (!parseArgs(args, arg::P<Locale>(TYPE_CLASSID(Locale), &locale)))
         {
             STATUS_CALL(calendar = Calendar::createInstance(*locale, status));
             return wrap_Calendar(calendar);
         }
         break;
       case 2:
-        if (!parseArgs(args, "PP",
-                       TYPE_CLASSID(TimeZone),
-                       TYPE_CLASSID(Locale),
-                       &tz, &locale))
+        if (!parseArgs(args,
+                       arg::P<TimeZone>(TYPE_CLASSID(TimeZone), &tz),
+                       arg::P<Locale>(TYPE_CLASSID(Locale), &locale)))
         {
             STATUS_CALL(calendar = Calendar::createInstance(*tz, *locale, status));
             return wrap_Calendar(calendar);
@@ -697,16 +707,16 @@ static PyObject *t_calendar_str(t_calendar *self)
     STATUS_CALL(date = self->object->getTime(status));
     STATUS_CALL(locale = self->object->getLocale(ULOC_VALID_LOCALE, status));
 
-    DateFormat *df = DateFormat::createDateTimeInstance(DateFormat::kDefault,
-                                                        DateFormat::kDefault,
-                                                        locale);
+    std::unique_ptr<DateFormat> df(
+        DateFormat::createDateTimeInstance(DateFormat::kDefault,
+                                           DateFormat::kDefault,
+                                           locale));
     df->format(date, u);
-    delete df;
 
     return PyUnicode_FromUnicodeString(&u);
 }
 
-DEFINE_RICHCMP(Calendar, t_calendar)
+DEFINE_RICHCMP__ARG__(Calendar, t_calendar)
 
 
 /* GregorianCalendar */
@@ -714,7 +724,7 @@ DEFINE_RICHCMP(Calendar, t_calendar)
 static int t_gregoriancalendar_init(t_gregoriancalendar *self,
                                     PyObject *args, PyObject *kwds)
 {
-    SimpleTimeZone *tz;
+    TimeZone *tz;
     Locale *locale;
     int year, month, date, hour, minute, second;
     GregorianCalendar *calendar;
@@ -726,14 +736,14 @@ static int t_gregoriancalendar_init(t_gregoriancalendar *self,
         self->flags = T_OWNED;
         break;
       case 1:
-        if (!parseArgs(args, "P", TYPE_CLASSID(TimeZone), &tz))
+        if (!parseArgs(args, arg::P<TimeZone>(TYPE_CLASSID(TimeZone), &tz)))
         {
             INT_STATUS_CALL(calendar = new GregorianCalendar(*tz, status));
             self->object = calendar;
             self->flags = T_OWNED;
             break;
         }
-        if (!parseArgs(args, "P", TYPE_CLASSID(Locale), &locale))
+        if (!parseArgs(args, arg::P<Locale>(TYPE_CLASSID(Locale), &locale)))
         {
             INT_STATUS_CALL(calendar = new GregorianCalendar(*locale, status));
             self->object = calendar;
@@ -743,10 +753,9 @@ static int t_gregoriancalendar_init(t_gregoriancalendar *self,
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
       case 2:
-        if (!parseArgs(args, "PP",
-                       TYPE_CLASSID(TimeZone),
-                       TYPE_CLASSID(Locale),
-                       &tz, &locale))
+        if (!parseArgs(args,
+                       arg::P<TimeZone>(TYPE_CLASSID(TimeZone), &tz),
+                       arg::P<Locale>(TYPE_CLASSID(Locale), &locale)))
         {
             INT_STATUS_CALL(calendar = new GregorianCalendar(*tz, *locale, status));
             self->object = calendar;
@@ -756,7 +765,7 @@ static int t_gregoriancalendar_init(t_gregoriancalendar *self,
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
       case 3:
-        if (!parseArgs(args, "iii", &year, &month, &date))
+        if (!parseArgs(args, arg::i(&year), arg::i(&month), arg::i(&date)))
         {
             INT_STATUS_CALL(calendar = new GregorianCalendar(year, month, date, status));
             self->object = calendar;
@@ -766,7 +775,7 @@ static int t_gregoriancalendar_init(t_gregoriancalendar *self,
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
       case 5:
-        if (!parseArgs(args, "iiiii", &year, &month, &date, &hour, &minute))
+        if (!parseArgs(args, arg::i(&year), arg::i(&month), arg::i(&date), arg::i(&hour), arg::i(&minute)))
         {
             INT_STATUS_CALL(calendar = new GregorianCalendar(year, month, date, hour, minute, status));
             self->object = calendar;
@@ -776,8 +785,7 @@ static int t_gregoriancalendar_init(t_gregoriancalendar *self,
         PyErr_SetArgsError((PyObject *) self, "__init__", args);
         return -1;
       case 6:
-        if (!parseArgs(args, "iiiiii", &year, &month, &date, &hour, &minute,
-                       &second))
+        if (!parseArgs(args, arg::i(&year), arg::i(&month), arg::i(&date), arg::i(&hour), arg::i(&minute), arg::i(&second)))
         {
             INT_STATUS_CALL(calendar = new GregorianCalendar(year, month, date, hour, minute, second, status));
             self->object = calendar;
@@ -807,7 +815,7 @@ static PyObject *t_gregoriancalendar_setGregorianChange(t_gregoriancalendar *sel
 {
     UDate date;
 
-    if (!parseArg(arg, "D", &date))
+    if (!parseArg(arg, arg::D(&date)))
     {
         STATUS_CALL(self->object->setGregorianChange(date, status));
         Py_RETURN_SELF();
@@ -820,7 +828,7 @@ static PyObject *t_gregoriancalendar_isLeapYear(t_gregoriancalendar *self, PyObj
 {
     int year, b;
 
-    if (!parseArg(arg, "i", &year))
+    if (!parseArg(arg, arg::i(&year)))
     {
         b = self->object->isLeapYear(year);
         Py_RETURN_BOOL(b);
