@@ -58,6 +58,9 @@ DECLARE_CONSTANTS_TYPE(UCPMapRangeOption)
 DECLARE_CONSTANTS_TYPE(UIdentifierType)
 DECLARE_CONSTANTS_TYPE(UIdentifierStatus)
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(76, 0, 0)
+DECLARE_CONSTANTS_TYPE(UIndicConjunctBreak)
+#endif
 
 /* Char */
 
@@ -1151,6 +1154,9 @@ void _init_char(PyObject *m)
     INSTALL_CONSTANTS_TYPE(UIdentifierType, m);
     INSTALL_CONSTANTS_TYPE(UIdentifierStatus, m);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(76, 0, 0)
+    INSTALL_CONSTANTS_TYPE(UIndicConjunctBreak, m);
+#endif    
 
     INSTALL_STRUCT(Char, m);
 #if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
@@ -1305,6 +1311,10 @@ void _init_char(PyObject *m)
 #if U_ICU_VERSION_HEX >= VERSION_HEX(75, 0, 0)
     INSTALL_ENUM(UProperty, "IDENTIFIER_STATUS", UCHAR_IDENTIFIER_STATUS);
     INSTALL_ENUM(UProperty, "IDENTIFIER_TYPE", UCHAR_IDENTIFIER_TYPE);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(76, 0, 0)
+    INSTALL_ENUM(UProperty, "INDIC_CONJUNCT_BREAK", UCHAR_INDIC_CONJUNCT_BREAK);
+    INSTALL_ENUM(UProperty, "MODIFIER_COMBINING_MARK", UCHAR_MODIFIER_COMBINING_MARK);
 #endif
     INSTALL_ENUM(UProperty, "INVALID_CODE", UCHAR_INVALID_CODE);
 
@@ -1735,6 +1745,16 @@ void _init_char(PyObject *m)
     INSTALL_ENUM(UBlockCode, "SYMBOLS_FOR_LEGACY_COMPUTING_SUPPLEMENT", UBLOCK_SYMBOLS_FOR_LEGACY_COMPUTING_SUPPLEMENT);
     INSTALL_ENUM(UBlockCode, "TODHRI", UBLOCK_TODHRI);
     INSTALL_ENUM(UBlockCode, "TULU_TIGALARI", UBLOCK_TULU_TIGALARI);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(78, 0, 0)
+    INSTALL_ENUM(UBlockCode, "BERIA_ERFE", UBLOCK_BERIA_ERFE);
+    INSTALL_ENUM(UBlockCode, "CJK_UNIFIED_IDEOGRAPHS_EXTENSION_J", UBLOCK_CJK_UNIFIED_IDEOGRAPHS_EXTENSION_J);
+    INSTALL_ENUM(UBlockCode, "MISCELLANEOUS_SYMBOLS_SUPPLEMENT", UBLOCK_MISCELLANEOUS_SYMBOLS_SUPPLEMENT);
+    INSTALL_ENUM(UBlockCode, "SHARADA_SUPPLEMENT", UBLOCK_SHARADA_SUPPLEMENT);
+    INSTALL_ENUM(UBlockCode, "SIDETIC", UBLOCK_SIDETIC);
+    INSTALL_ENUM(UBlockCode, "TAI_YO", UBLOCK_TAI_YO);
+    INSTALL_ENUM(UBlockCode, "TANGUT_COMPONENTS_SUPPLEMENT", UBLOCK_TANGUT_COMPONENTS_SUPPLEMENT);
+    INSTALL_ENUM(UBlockCode, "TOLONG_SIKI", UBLOCK_TOLONG_SIKI);
 #endif
     INSTALL_ENUM(UBlockCode, "INVALID_CODE", UBLOCK_INVALID_CODE);
 
@@ -1812,6 +1832,13 @@ void _init_char(PyObject *m)
 #if U_ICU_VERSION_HEX >= VERSION_HEX(76, 0, 0)
     INSTALL_ENUM(UIndicSyllabicCategory, "REORDERING_KILLER", U_INSC_REORDERING_KILLER);
 #endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(76, 0, 0)
+    INSTALL_ENUM(UIndicConjunctBreak, "CONSONANT", U_INCB_CONSONANT);
+    INSTALL_ENUM(UIndicConjunctBreak, "EXTEND", U_INCB_EXTEND);
+    INSTALL_ENUM(UIndicConjunctBreak, "LINKER", U_INCB_LINKER);
+    INSTALL_ENUM(UIndicConjunctBreak, "NONE", U_INCB_NONE);
+#endif    
     
     INSTALL_ENUM(UVerticalOrientation, "ROTATED", U_VO_ROTATED);
     INSTALL_ENUM(UVerticalOrientation, "TRANSFORMED_ROTATED", U_VO_TRANSFORMED_ROTATED);
@@ -1923,6 +1950,9 @@ void _init_char(PyObject *m)
 #if U_ICU_VERSION_HEX >= VERSION_HEX(76, 0, 0)
     INSTALL_ENUM(UJoiningGroup, "KASHMIRI_YEH", U_JG_KASHMIRI_YEH);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(78, 0, 0)
+    INSTALL_ENUM(UJoiningGroup, "THIN_NOON", U_JG_THIN_NOON);
+#endif
 
     INSTALL_ENUM(ULineBreak, "INSEPARABLE", U_LB_INSEPARABLE);
     INSTALL_ENUM(ULineBreak, "NEXT_LINE", U_LB_NEXT_LINE);
@@ -1953,6 +1983,9 @@ void _init_char(PyObject *m)
     INSTALL_ENUM(ULineBreak, "VIRAMA_FINAL", U_LB_VIRAMA_FINAL);
     INSTALL_ENUM(ULineBreak, "VIRAMA", U_LB_VIRAMA);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(78, 0, 0)
+    INSTALL_ENUM(ULineBreak, "UNAMBIGUOUS_HYPHEN", U_LB_UNAMBIGUOUS_HYPHEN);
+#endif
 
     INSTALL_ENUM(UGraphemeClusterBreak, "OTHER", U_GCB_OTHER);
     INSTALL_ENUM(UGraphemeClusterBreak, "CONTROL", U_GCB_CONTROL);
