@@ -34,6 +34,8 @@
 #include "arg.h"
 
 DECLARE_CONSTANTS_TYPE(UCollationResult)
+DECLARE_CONSTANTS_TYPE(ECollationStrength)
+DECLARE_CONSTANTS_TYPE(EComparisonResult)
 DECLARE_CONSTANTS_TYPE(UCollAttribute)
 DECLARE_CONSTANTS_TYPE(UCollAttributeValue)
 #if U_ICU_VERSION_HEX >= 0x04080000
@@ -536,7 +538,7 @@ static PyObject *t_collator_getSortKey(t_collator *self, PyObject *args)
     UnicodeString *u;
     UnicodeString _u;
     int len, size;
-    uint8_t *buf;
+    std::unique_ptr<uint8_t[]> buf;
     PyObject *key;
 
     switch (PyTuple_Size(args)) {
@@ -544,21 +546,18 @@ static PyObject *t_collator_getSortKey(t_collator *self, PyObject *args)
         if (!parseArgs(args, arg::S(&u, &_u)))
         {
             len = u->length() * 4 + 8;
-            buf = (uint8_t *) malloc(len);
+            buf.reset(new uint8_t[len]);
           retry:
-            if (buf == NULL)
+            if (!buf.get())
                 return PyErr_NoMemory();
 
-            size = self->object->getSortKey(*u, buf, len);
+            size = self->object->getSortKey(*u, buf.get(), len);
             if (size <= len)
-            {
-                key = PyBytes_FromStringAndSize((char *) buf, size);
-                free(buf);
-            }
+                key = PyBytes_FromStringAndSize((char *) buf.get(), size);
             else
             {
                 len = size;
-                buf = (uint8_t *) realloc(buf, len);
+                buf.reset(new uint8_t[len]);
                 goto retry;
             }
 
@@ -568,13 +567,12 @@ static PyObject *t_collator_getSortKey(t_collator *self, PyObject *args)
       case 2:
         if (!parseArgs(args, arg::S(&u, &_u), arg::i(&len)))
         {
-            buf = (uint8_t *) calloc(len, 1);
-            if (buf == NULL)
+            buf.reset(new uint8_t[len]);
+            if (!buf.get())
                 return PyErr_NoMemory();
 
-            len = self->object->getSortKey(*u, buf, len);
-            key = PyBytes_FromStringAndSize((char *) buf, len);
-            free(buf);
+            len = self->object->getSortKey(*u, buf.get(), len);
+            key = PyBytes_FromStringAndSize((char *) buf.get(), len);
 
             return key;
         }
@@ -1413,6 +1411,8 @@ void _init_collator(PyObject *m)
     ImmutableIndexType_.tp_as_sequence = &t_immutableindex_as_sequence;
 #endif
     INSTALL_CONSTANTS_TYPE(UCollationResult, m);
+    INSTALL_CONSTANTS_TYPE(ECollationStrength, m);
+    INSTALL_CONSTANTS_TYPE(EComparisonResult, m);
     INSTALL_CONSTANTS_TYPE(UCollAttribute, m);
     INSTALL_CONSTANTS_TYPE(UCollAttributeValue, m);
     REGISTER_TYPE(CollationKey, m);
@@ -1430,6 +1430,16 @@ void _init_collator(PyObject *m)
     INSTALL_ENUM(UCollationResult, "EQUAL", UCOL_EQUAL);
     INSTALL_ENUM(UCollationResult, "GREATER", UCOL_GREATER);
 
+    INSTALL_ENUM(ECollationStrength, "PRIMARY", UCOL_PRIMARY);
+    INSTALL_ENUM(ECollationStrength, "SECONDARY", UCOL_SECONDARY);
+    INSTALL_ENUM(ECollationStrength, "TERTIARY", UCOL_TERTIARY);
+    INSTALL_ENUM(ECollationStrength, "QUATERNARY", UCOL_QUATERNARY);
+    INSTALL_ENUM(ECollationStrength, "IDENTICAL", UCOL_IDENTICAL);
+
+    INSTALL_ENUM(EComparisonResult, "LESS", UCOL_LESS);
+    INSTALL_ENUM(EComparisonResult, "EQUAL", UCOL_EQUAL);
+    INSTALL_ENUM(EComparisonResult, "GREATER", UCOL_GREATER);
+
     INSTALL_ENUM(UCollAttribute, "FRENCH_COLLATION", UCOL_FRENCH_COLLATION);
     INSTALL_ENUM(UCollAttribute, "ALTERNATE_HANDLING", UCOL_ALTERNATE_HANDLING);
     INSTALL_ENUM(UCollAttribute, "CASE_FIRST", UCOL_CASE_FIRST);
