@@ -594,6 +594,13 @@ void _init_script(PyObject *m)
 #if U_ICU_VERSION_HEX >= VERSION_HEX(75, 0, 0)
     INSTALL_ENUM(UScriptCode, "ARABIC_NASTALIQ", USCRIPT_ARABIC_NASTALIQ);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(78, 0, 0)
+    INSTALL_ENUM(UScriptCode, "BERIA_ERFE", USCRIPT_BERIA_ERFE);
+    INSTALL_ENUM(UScriptCode, "SIDETIC", USCRIPT_SIDETIC);
+    INSTALL_ENUM(UScriptCode, "TAI_YO", USCRIPT_TAI_YO);
+    INSTALL_ENUM(UScriptCode, "TOLONG_SIKI", USCRIPT_TOLONG_SIKI);
+    INSTALL_ENUM(UScriptCode, "TRADITIONAL_HAN_WITH_LATIN", USCRIPT_TRADITIONAL_HAN_WITH_LATIN);
+#endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
     INSTALL_ENUM(UScriptUsage, "NOT_ENCODED", USCRIPT_USAGE_NOT_ENCODED);
